## So much is identical here to gspinbutton, we should make a class to derive these from -- another day.


setClass("gSliderRGtk",
         contains="gComponentRGtk",
         prototype=prototype(new("gComponentRGtk"))
         )

setMethod(".gslider",
          signature(toolkit="guiWidgetsToolkitRGtk2"),
          function(toolkit,
                   from=0, to=100, by = 1,
                   value=from,
                   horizontal=TRUE,
                   handler=NULL, action=NULL,
                   container=NULL, ...) {

            force(toolkit)
            
            if (horizontal)
              scale <- gtkHScaleNewWithRange(from, to, by)
            else scale <- gtkVScaleNewWithRange(from, to, by)
            scale$setValue(value)

            obj <- as.gWidgetsRGtk2(scale)

            
            if (!is.null(container)) {
              if(is.logical(container) && container == TRUE)
                container = gwindow(visible=TRUE)
              add(container, obj,...)
            }
            
            if (!is.null(handler))  {
              id = addhandlerchanged(obj, handler, action)
            }
            
            invisible(obj)
          })

as.gWidgetsRGtk2.GtkHScale <- as.gWidgetsRGtk2.GtkVScale <-
  function(widget,...) {
    obj <- new("gSliderRGtk",block=widget, widget=widget,
               toolkit=guiToolkit("RGtk2"))
    return(obj)
  }

### methods
setMethod(".svalue",
          signature(toolkit="guiWidgetsToolkitRGtk2",obj="gSliderRGtk"),
          function(obj, toolkit, index=NULL, drop=NULL, ...) {
            obj@widget$getValue()
          })

setReplaceMethod(".svalue",
                 signature(toolkit="guiWidgetsToolkitRGtk2",obj="gSliderRGtk"),
                 function(obj, toolkit, index=NULL, ..., value) {
                   obj@widget$setValue(value)
                   return(obj)
                 })


## Method to replace values of sping button
setReplaceMethod("[",
                 signature(x="gSliderRGtk"),
                 function(x, i, j,..., value) {
                   .leftBracket(x, x@toolkit, i, j, ...) <- value
                   return(x)
                 })

setReplaceMethod(".leftBracket",
          signature(toolkit="guiWidgetsToolkitRGtk2",x="gSliderRGtk"),
          function(x, toolkit, i, j, ..., value) {
            obj <- x
            widget <- getWidget(obj)

            ## check that value is a regular sequence
            if(length(value) <=1) {
              warning("Can only assign a vector with equal steps, as produced by seq")
              return(obj)
            }
            if(length(value) > 2 &&
               !all.equal(diff(diff(value)), rep(0, length(value) - 2))) {
              warning("Can only assign a vector with equal steps, as produced by seq")
              return(obj)
            }
            ## get current value, increment
            curValue <- svalue(obj)
            inc <- head(diff(value), n=1)

            widget$setRange(min(value), max(value))
            widget$setIncrements(inc, inc) # button 1, button 2
            widget$setValue(curValue)


            ## all done
            return(obj)
          })




### handlers
setMethod(".addhandlerchanged",
          signature(toolkit="guiWidgetsToolkitRGtk2",obj="gSliderRGtk"),
          function(obj, toolkit, handler, action=NULL, ...) {
            addhandler(obj, "value-changed", handler, action,...)
          })
